@@ -45,7 +45,6 @@ var (
 	goos        = runtime.GOOS
 	goarch      = runtime.GOARCH
 	target      = fmt.Sprintf("%s/%s", goos, goarch)
-	nogcc       = goos == "windows" && goarch == "arm64" // We have no 32b mingw-gcc binary targeting windows/amd64 bit yet.
 
 	oTrace = flag.Bool("trc", false, "Print tested paths.")
 )
@@ -590,10 +589,6 @@ func defaultCfg() *Config {
 }
 
 func TestCPPExpand(t *testing.T) {
-	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
-		t.Skip("TODO")
-	}
-
 	testCPPExpand(t, "testdata/cpp-expand/", nil, true)
 }
 
@@ -709,18 +704,10 @@ func testCPPExpand(t *testing.T, dir string, blacklist map[string]struct{}, fake
 }
 
 func TestPreprocess(t *testing.T) {
-	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
-		t.Skip("TODO")
-	}
-
 	testCPPExpand(t, "testdata/preprocess/", nil, true)
 }
 
 func TestTCCExpand(t *testing.T) {
-	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
-		t.Skip("TODO")
-	}
-
 	testCPPExpand(t, "testdata/tcc-0.9.27/tests/pp/", map[string]struct{}{
 		"11.c": {}, // https://gcc.gnu.org/onlinedocs/gcc/Variadic-Macros.html#Variadic-Macros
 		"16.c": {}, // We don't produce warnings on macro redefinition.
@@ -728,10 +715,6 @@ func TestTCCExpand(t *testing.T) {
 }
 
 func TestInclude(t *testing.T) {
-	if runtime.GOOS == "windows" && runtime.GOARCH == "arm64" {
-		t.Skip("TODO")
-	}
-
 	testCPPExpand(t, "testdata/include/", nil, false)
 }
 
@@ -1135,24 +1118,33 @@ func TestParse(t *testing.T) {
 	case "freebsd/amd64":
 		blacklistDebian["mandelbrot-6.c"] = struct{}{}
 		blacklistDebian["spectral-norm-5.c"] = struct{}{}
+	case "darwin/amd64":
+		blacklistDebian["fannkuchredux-4.c"] = struct{}{}
+		blacklistDebian["mandelbrot-6.c"] = struct{}{}
 	}
+	type bm map[string]struct{}
 	var files, ok, skip, fails int32
 	for _, v := range []struct {
-		cfg       *Config
-		dir       string
-		blacklist map[string]struct{}
+		cfg             *Config
+		dir             string
+		blacklist       map[string]struct{}
+		targetBlacklist map[string]struct{}
 	}{
-		{cfg, "CompCert-3.6/test/c", blacklistCompCert},
-		{cfg, "ccgo", nil},
-		{cfg, "gcc-9.1.0/gcc/testsuite/gcc.c-torture", blacklistGCC},
-		{cfg, "github.com/AbsInt/CompCert/test/c", blacklistCompCert},
-		{cfg, "github.com/cxgo", nil},
-		{cfg, "github.com/gcc-mirror/gcc/gcc/testsuite", blacklistGCC},
-		{cfg, "github.com/vnmakarov", nil},
-		{cfg, "sqlite-amalgamation", nil},
-		{cfg, "tcc-0.9.27/tests/tests2", nil},
-		{cfg, "benchmarksgame-team.pages.debian.net", blacklistDebian},
+		{cfg, "CompCert-3.6/test/c", blacklistCompCert, nil},
+		{cfg, "ccgo", nil, nil},
+		{cfg, "gcc-9.1.0/gcc/testsuite/gcc.c-torture", blacklistGCC, nil},
+		{cfg, "github.com/AbsInt/CompCert/test/c", blacklistCompCert, nil},
+		{cfg, "github.com/cxgo", nil, nil},
+		{cfg, "github.com/gcc-mirror/gcc/gcc/testsuite", blacklistGCC, nil},
+		{cfg, "github.com/vnmakarov", nil, nil},
+		{cfg, "sqlite-amalgamation", nil, bm{"windows/amd64": {}}},
+		{cfg, "tcc-0.9.27/tests/tests2", nil, nil},
+		{cfg, "benchmarksgame-team.pages.debian.net", blacklistDebian, nil},
 	} {
+		if _, ok := v.targetBlacklist[target]; ok {
+			continue
+		}
+
 		t.Run(v.dir, func(t *testing.T) {
 			f, o, s, n := testParse(t, v.cfg, "assets/"+v.dir, v.blacklist)
 			files += f
@@ -1375,10 +1367,6 @@ func testTranslateBug(t *testing.T, dir string, blacklist map[string]struct{}) {
 }
 
 func TestTranslate(t *testing.T) {
-	if nogcc {
-		t.Skip()
-	}
-
 	cfg := defaultCfg()
 	cfg.SysIncludePaths = append(cfg.SysIncludePaths, "Include") // benchmarksgame
 	cfg.FS = cfs
@@ -1418,11 +1406,16 @@ func TestTranslate(t *testing.T) {
 		blacklistGCC["pr93213.c"] = struct{}{}     //TODO
 		blacklistGCC["pr98474.c"] = struct{}{}     //TODO
 		blacklistMakarov["setjmp2.c"] = struct{}{} //TODO
+	case "windows/amd64":
+    		blacklistGCC["pr30704.c"] = struct{}{} //TODO
 	case "windows/386":
 		blacklistGCC["pr30704.c"] = struct{}{} // '__int128' is not supported on this target
 	case "freebsd/amd64":
 		blacklistDebian["mandelbrot-6.c"] = struct{}{}
 		blacklistDebian["spectral-norm-5.c"] = struct{}{}
+	case "darwin/amd64":
+		blacklistDebian["fannkuchredux-4.c"] = struct{}{}
+		blacklistDebian["mandelbrot-6.c"] = struct{}{}
 	}
 	var files, ok, skip, fails int32
 	for _, v := range []struct {
@@ -1441,6 +1434,10 @@ func TestTranslate(t *testing.T) {
 		{cfg, "tcc-0.9.27/tests/tests2", nil},
 		{cfg, "benchmarksgame-team.pages.debian.net", blacklistDebian},
 	} {
+		if target == "windows/amd64" && strings.Contains(v.dir, "sqlite") {
+			continue
+		}
+
 		t.Run(v.dir, func(t *testing.T) {
 			f, o, s, n := testTranslate(t, v.cfg, "assets/"+v.dir, v.blacklist)
 			files += f
