@@ -5,6 +5,7 @@
 package cc // import "modernc.org/cc/v4"
 
 import (
+	"fmt"
 	"math/big"
 )
 
@@ -129,11 +130,19 @@ type UTF32StringValue []rune
 func (n UTF32StringValue) Convert(to Type) Value { return convert(n, to) }
 
 func (n *ConstantExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	n.val = n.ConditionalExpression.eval(c, mode)
 	return n.Value()
 }
 
 func (n *ConditionalExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -156,6 +165,10 @@ func (n *ConditionalExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *LogicalOrExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -183,6 +196,10 @@ func (n *LogicalOrExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *LogicalAndExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -210,6 +227,10 @@ func (n *LogicalAndExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *InclusiveOrExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -223,7 +244,7 @@ func (n *InclusiveOrExpression) eval(c *ctx, mode flags) (r Value) {
 		case *UnknownValue:
 			// nop
 		case Int64Value:
-			switch y := convert(n.InclusiveOrExpression.eval(c, mode), n.Type()).(type) {
+			switch y := convert(n.ExclusiveOrExpression.eval(c, mode), n.Type()).(type) {
 			case *UnknownValue:
 				// nop
 			case Int64Value:
@@ -232,7 +253,7 @@ func (n *InclusiveOrExpression) eval(c *ctx, mode flags) (r Value) {
 				c.errors.add(errorf("TODO %v TYPE %T", n.Case, y))
 			}
 		case UInt64Value:
-			switch y := convert(n.InclusiveOrExpression.eval(c, mode), n.Type()).(type) {
+			switch y := convert(n.ExclusiveOrExpression.eval(c, mode), n.Type()).(type) {
 			case *UnknownValue:
 				// nop
 			case UInt64Value:
@@ -250,6 +271,10 @@ func (n *InclusiveOrExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *ExclusiveOrExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -290,6 +315,10 @@ func (n *ExclusiveOrExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *AndExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -330,6 +359,10 @@ func (n *AndExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *EqualityExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -354,6 +387,8 @@ func (n *EqualityExpression) eval(c *ctx, mode flags) (r Value) {
 				// ok
 			case Int64Value:
 				n.val = bool2int(x == y)
+			case UInt64Value:
+				n.val = bool2int(x == Int64Value(y))
 			default:
 				c.errors.add(errorf("TODO %v TYPE %T", n.Case, y))
 			}
@@ -363,6 +398,8 @@ func (n *EqualityExpression) eval(c *ctx, mode flags) (r Value) {
 				// ok
 			case UInt64Value:
 				n.val = bool2int(x == y)
+			case Int64Value:
+				n.val = bool2int(x == UInt64Value(y))
 			default:
 				c.errors.add(errorf("TODO %v TYPE %T", n.Case, y))
 			}
@@ -411,6 +448,10 @@ func (n *EqualityExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *RelationalExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -590,6 +631,10 @@ func (n *RelationalExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *ShiftExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -607,6 +652,11 @@ func (n *ShiftExpression) eval(c *ctx, mode flags) (r Value) {
 			case *UnknownValue:
 				// nop
 			case Int64Value:
+				if y < 0 {
+					c.errors.add(fmt.Errorf("%v: negative shift amount: %v << %v", position(n), x, y))
+					break
+				}
+
 				n.val = convert(x<<y, n.Type())
 			case UInt64Value:
 				n.val = convert(x<<y, n.Type())
@@ -618,6 +668,11 @@ func (n *ShiftExpression) eval(c *ctx, mode flags) (r Value) {
 			case *UnknownValue:
 				// nop
 			case Int64Value:
+				if y < 0 {
+					c.errors.add(fmt.Errorf("%v: negative shift amount: %v << %v", position(n), x, y))
+					break
+				}
+
 				n.val = convert(x<<y, n.Type())
 			case UInt64Value:
 				n.val = convert(x<<y, n.Type())
@@ -636,6 +691,11 @@ func (n *ShiftExpression) eval(c *ctx, mode flags) (r Value) {
 			case *UnknownValue:
 				// nop
 			case Int64Value:
+				if y < 0 {
+					c.errors.add(fmt.Errorf("%v: negative shift amount: %v >> %v", position(n), x, y))
+					break
+				}
+
 				n.val = convert(x>>y, n.Type())
 			case UInt64Value:
 				n.val = convert(x>>y, n.Type())
@@ -647,6 +707,11 @@ func (n *ShiftExpression) eval(c *ctx, mode flags) (r Value) {
 			case *UnknownValue:
 				// nop
 			case Int64Value:
+				if y < 0 {
+					c.errors.add(fmt.Errorf("%v: negative shift amount: %v >> %v", position(n), x, y))
+					break
+				}
+
 				n.val = convert(x>>y, n.Type())
 			case UInt64Value:
 				n.val = convert(x>>y, n.Type())
@@ -663,6 +728,10 @@ func (n *ShiftExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *AdditiveExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -730,6 +799,10 @@ func (n *AdditiveExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *MultiplicativeExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -832,6 +905,10 @@ func (n *MultiplicativeExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *CastExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		switch n.Case {
 		case CastExpressionUnary: // UnaryExpression
@@ -856,6 +933,10 @@ func (n *CastExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *UnaryExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		switch n.Case {
 		case UnaryExpressionPostfix: // PostfixExpression
@@ -865,7 +946,7 @@ func (n *UnaryExpression) eval(c *ctx, mode flags) (r Value) {
 		case UnaryExpressionDec: // "--" UnaryExpression
 			c.errors.add(errorf("TODO %T %v", n, n.Case))
 		case UnaryExpressionAddrof: // '&' CastExpression
-			c.errors.add(errorf("TODO %T %v", n, n.Case))
+			// ok
 		case UnaryExpressionDeref: // '*' CastExpression
 			switch x := n.CastExpression.eval(c, mode.del(addrOf)).(type) {
 			case *UnknownValue:
@@ -990,6 +1071,10 @@ func (n *UnaryExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *PostfixExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		switch n.Case {
 		case PostfixExpressionPrimary: // PrimaryExpression
@@ -1190,6 +1275,10 @@ func (n *PostfixExpression) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *PrimaryExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		switch n.Case {
 		case PrimaryExpressionIdent: // IDENTIFIER
@@ -1255,6 +1344,10 @@ func (n *ExpressionList) eval(c *ctx, mode flags) (r Value) {
 		return Unknown
 	}
 
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	n0 := n
 	for ; n != nil; n = n.ExpressionList {
 		n0.typ = n.AssignmentExpression.Type()
@@ -1264,6 +1357,10 @@ func (n *ExpressionList) eval(c *ctx, mode flags) (r Value) {
 }
 
 func (n *AssignmentExpression) eval(c *ctx, mode flags) (r Value) {
+	if n.val != nil && n.val != Unknown {
+		return n.Value()
+	}
+
 	if mode.has(addrOf) {
 		c.errors.add(errorf("TODO %v %v", n.Case, mode.has(addrOf)))
 		return n.Value()
@@ -1301,6 +1398,8 @@ func isZero(v Value) bool {
 		return x == 0
 	case UInt64Value:
 		return x == 0
+	case StringValue:
+		return false
 	default:
 		panic(todo("%T", x))
 	}
@@ -1314,6 +1413,8 @@ func isNonzero(v Value) bool {
 		return x != 0
 	case UInt64Value:
 		return x != 0
+	case StringValue:
+		return true
 	default:
 		panic(todo("%T", x))
 	}
